#!/usr/bin/python
#	This script analyse the data acquired by par-logger.ko and
#	saved to files by logdata.
#	Copyright (C) 2005  Kristian Benoit, Opersys inc.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


import sys
import getopt
import re

def average(values):
	"""Computes the averages of the list values"""
	return sum(values,0.0)/len(values)

def stddev(values):
	"""Computes the standatd deviation of the list values"""
	mu = average(values)
	sum = 0.0
	for val in values:
		sum += pow(val-mu,2)
	return pow(sum / len(values), 0.5)

delta = []
delta_timer = []
print_delta = False
print_summary = True

cpuinfo = open("/proc/cpuinfo")
cpuspeed = cpuinfo.readline()

while re.search("MHz", cpuspeed) == None:
	cpuspeed = cpuinfo.readline()

cpuspeed = float(re.sub('^(\D*)([0-9.]*)(\D*)$',r'\2', cpuspeed))
divisor = cpuspeed
unity = " us"

args,filenames = getopt.gnu_getopt(sys.argv[1:],"dascf:")

if len(filenames) == 0:
	filenames = ["data"]

for p in args:
	if p[0] == "-d": # Show only the deltas
		print_summary=False
		print_delta=True
	elif p[0] == "-a": # Show all (the summary and the deltas)
		print_summary=True
		print_delta=True
	elif p[0] == "-s": # Show only the summary
		print_summary=True
		print_delta=False
	elif p[0] == "-c": # Show as cycles instead of us
		divisor = 1
		unity = " cycles"
	elif p[0] == "-f": # Set the cpu frequency
		divisor = float(p[1])
		cpuspeed = divisor
		unity = " us"


bad_delta=0
for filename in filenames:
	datafile = open(filename, 'r')

#	print filename
	old_timer=False
	type1=datafile.read(1)
	line1=datafile.readline()
	type2=datafile.read(1)
	line2=datafile.readline()
	while type1 != "t" and type2 != "p" and line2 != "":
		type1 = type2
		line1 = line2
		type2=datafile.read(1)
		line2=datafile.readline()
		bad_delta += 1
		
	while line2 != "":
		int1 = int(line1)
		int2 = int(line2)
		delta += [int2-int1]
		if (old_timer):
			delta_timer += [int1-old_timer]
		old_timer = int1

		type1=datafile.read(1)
		line1=datafile.readline()
		type2=datafile.read(1)
		line2=datafile.readline()
		while type1 != "t" and type2 != "p" and line2 != "":
			type1 = type2
			line1 = line2
			type2=datafile.read(1)
			line2=datafile.readline()
			old_timer = False
			bad_delta += 1

	datafile.close()

if print_delta:
	for d in delta:
		print d/divisor

if print_summary:
	print
	print "Sanity check (freq ~ HZ) : " + str(average(delta_timer)/cpuspeed) +" +- " + str(stddev(delta_timer)/cpuspeed) + " us"
	print
	print "Number of delta : " + str(len(delta)) + " deltas"
	print "Number of bad delta : " + str(bad_delta) + " deltas"
	print "Minimum delta : " + str(min(delta)/divisor) + unity
	print "Maximum delta : " + str(max(delta)/divisor) + unity
	print "Average : " + str(average(delta)/divisor) + unity
	print "Standard deviation : " + str(stddev(delta)/divisor) + unity
	print

