#!/usr/bin/python
#	This script automate/synchronise the different tests for realtime in linux.
#	Copyright (C) 2005  Kristian Benoit, Opersys inc.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


#import thread
import threading
import time
import os
import re

# Global constant change to your need
target_ip = "192.168.172.50"
logger_ip = "192.168.172.49"
hd_script = "./hd_script" # On target
lmbench_file = "test/lmbench-2.0.4/results/i686-pc-linux-gnu/localhost.localdomain.0" #The file is on the target
analyse_test_files = "results/*" # The directory is on the logger machine
user = "stage"
results_tree = "results"
sleep_time = 3.5*60

# Definitions : Threads that start a test.

# These 2 tests control the duration of the tests.
def start_lmbench(event, testname, testdir):
	t1 = time.time()
	os.system("ssh " + user + "@" + target_ip + " ' cd test/lmbench-2.0.4/src ; make rerun'")
	delta_t = time.time()-t1
	event.set()

	print "Cleaning lmbench"
	timefile = open(os.path.join (testdir, "lmbench.time"), "a")
	timefile.write(str(delta_t) + "\n")
	timefile.close()
	os.system("scp " + user + "@" + target_ip + ":" + lmbench_file + " " + os.path.join(testdir, testname + ".lmbench"))
	os.system("ssh " + user + "@" + target_ip + " rm -f " + lmbench_file)

def start_sleeping (event, testname, testdir):
	time.sleep(sleep_time)
	event.set()

# Those are wait for the previous to be done.
def start_logging (event, testname, testdir):
	os.system("ssh " + user + "@" + logger_ip + " 'sudo /sbin/insmod par-log.ko'")

	event.wait()

	print "Cleaning IRQ"
	os.system("ssh " + user + "@" + logger_ip + " '" + log_script_name + " " + testname + "'")
	analyse_dir = os.path.join(testdir, "analyse")
	_mkdir(analyse_dir)
	os.system("scp " + user + "@" + logger_ip + ":" + analyse_test_files + " " + analyse_dir)
	os.system("ssh " + user + "@" + logger_ip + " 'rm -f " + analyse_test_files + "'")
	os.system("ssh " + user + "@" + logger_ip + " sudo /sbin/rmmod par-log")

def start_pinging (event, testname, testdir):
	pid_file = os.popen("sudo ping -f " + target_ip + " > /dev/null & echo $!", "r")
	pid = int(pid_file.readline())
	event.wait()
	os.system("sudo kill -s 9 " + str(pid))
	pid_file.close()

def start_hd (event, testname, testdir):
	pid_file = os.popen("ssh " + user + "@" + target_ip + " '" + hd_script + "& echo $!'", "r")
	pid = int(pid_file.readline())
	event.wait()
	os.system("ssh " + user + "@" + target_ip + " 'kill " + str(pid) + "'")
	pid_file.close()


# Comment the test you dont want or add your.
tests_list = [{"name" : "lmbench-irq_test",            "nbpass" : 3, "timed_test" : start_lmbench,   "other_tests" : [start_logging]},
			  {"name" : "lmbench-ping_flood",          "nbpass" : 3, "timed_test" : start_lmbench,   "other_tests" : [start_pinging]},
		      {"name" : "lmbench",                     "nbpass" : 3, "timed_test" : start_lmbench,   "other_tests" : []},
			  {"name" : "lmbench-irq_test-ping_flood", "nbpass" : 3, "timed_test" : start_lmbench,   "other_tests" : [start_pinging, start_logging]},
			  {"name" : "irq_test",                    "nbpass" : 3, "timed_test" : start_sleeping,  "other_tests" : [start_logging]},
			  {"name" : "irq_test-ping_flood",         "nbpass" : 3, "timed_test" : start_sleeping,  "other_tests" : [start_pinging, start_logging]},
			  {"name" : "lmbench-hd",                  "nbpass" : 3, "timed_test" : start_lmbench,   "other_tests" : [start_hd, start_logging]},
			  ]

# From http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/82465
def _mkdir(newdir):
	"""works the way a good mkdir should :)
	- already exists, silently complete
	- regular file in the way, raise an exception
	- parent directory(ies) does not exist, make them as well
	"""
	if os.path.isdir(newdir):
		pass
	elif os.path.isfile(newdir):
		raise OSError("a file with the same name as the desired " \
					  "dir, '%s', already exists." % newdir)
	else:
		head, tail = os.path.split(newdir)
		if head and not os.path.isdir(head):
			_mkdir(head)
		#print "_mkdir %s" % repr(newdir)
		if tail:
			os.mkdir(newdir)

# Here we begin the tests.
os.system ("ssh " + user + "@" + target_ip + " sudo /sbin/insmod par-test.ko")
version_file = os.popen ("ssh " + user + "@" + target_ip + " uname -r", "r")
kversion = re.sub("\n", "", version_file.readline())
results_tree = os.path.join(results_tree, kversion)
_mkdir(results_tree)
print "Results will go in " + results_tree
os.system("scp " + user + "@" + target_ip + ":/lib/modules/" + kversion  + "/build/.config " + results_tree + "/config")

event = threading.Event()
for test_desc in tests_list:
	test_dir = os.path.join(results_tree, test_desc["name"])
	_mkdir(test_dir)
	for i in range(test_desc["nbpass"]):
		test_pass_dir = os.path.join(test_dir, "pass" + str(i))
		_mkdir(test_pass_dir)
		event.clear()
		print "starting " + test_desc["name"] + str(i)

		for test in test_desc["other_tests"]:
			print "         starting " + str(test) + "..."
			mythread = threading.Thread(target=test, args=(event, test_desc["name"], test_pass_dir))
			mythread.start()

		print "         starting " + str(test_desc["timed_test"]) + "..."
		mythread = threading.Thread(target=test_desc["timed_test"], args=(event, test_desc["name"], test_pass_dir))
		mythread.start()

		print
		print "waiting for threads to be done..."
		mythread.join()
		while threading.activeCount() != 1:
			pass # make sure the other threads are done.
		print "... threads done"
		print
		print

os.system ("ssh " + user + "@" + target_ip + " sudo /sbin/rmmod par-test")
